# coding:utf-8
import datetime as dt
from bs4 import BeautifulSoup

from src.browser import Browser
from src.utils.color import Color


class AttendanceManager(object):

    def __init__(self, headless=True):
        self.headless = headless
        self.browser = Browser(headless=headless)
        self.driver = self.browser.driver

    def confirm(self, method):
        if self.open() and method in dir(self):
            getattr(self, method)()
            self.exit()

    def open(self):
        return self.browser.open_attendance()

    def history(self):
        timetable = self.get_attendance_timetable(dt.datetime.now().day)
        self.print_timetable(timetable)

    def print_timetable(self, timetable):
        texts = [
            Color.get_colored(Color.BOLD, '出勤:     ')
            + ', '.join(timetable[0]),
            Color.get_colored(Color.BOLD, '退勤:     ')
            + ', '.join(timetable[1]),
            Color.get_colored(Color.BOLD, '休憩開始: ')
            + ', '.join(timetable[2]),
            Color.get_colored(Color.BOLD, '休憩終了: ')
            + ', '.join(timetable[3])
        ]
        print('================================')
        for text in texts:
            print(text)
        print('================================')

    def get_attendance_timetable(self, day) -> list:
        timetable = self.get_attendance(day)
        for i, t in enumerate(timetable):
            timetable[i] = [t for t in t.strings]

        return timetable

    def get_attendance(self, day) -> list:
        html = self.driver.page_source.encode('utf-8')
        soup = BeautifulSoup(html, 'html.parser')
        attendances = soup.find_all('td', class_='column-attendance')

        attendance_array = []
        time = 0
        while time < len(attendances):
            attendance_array.append(attendances[time:time+4])
            time += 4

        return attendance_array[day-1]

    def exit(self):
        self.driver.close()



# coding:utf-8
import sys
import fire

from src.user_info_manager import UserInfoManager
from src.stamp_manager import StampManager
from src.attendance_manager import AttendanceManager


class Dakoker(object):
    """
    Dakoker:                MFクラウド勤怠向けの打刻ツールです\n

    dakoker start:          出勤の打刻をします\n
    dakoker end:            退勤の打刻をします\n
    dakoker start_break:    休憩開始の打刻をします\n
    dakoker end_break:      休憩終了の打刻をします\n
    dakoker history:        当日の勤怠状況を確認できます\n
    dakoker open:           MFクラウド勤怠ページを開きます\n
    dakoker clear:          ユーザーログイン情報のローカルキャッシュをクリアします\n
    """

    def start(self):
        """
        dakoker start:          出勤の打刻をします
        """
        StampManager().stamp(sys._getframe().f_code.co_name)

    def end(self):
        """
        dakoker end:            退勤の打刻をします
        """
        StampManager().stamp(sys._getframe().f_code.co_name)

    def start_break(self):
        """
        dakoker start_break:    休憩開始の打刻をします
        """
        StampManager().stamp(sys._getframe().f_code.co_name)

    def end_break(self):
        """
        dakoker end_break:      休憩終了の打刻をします
        """
        StampManager().stamp(sys._getframe().f_code.co_name)

    def history(self):
        """
        dakoker history:        当日の勤怠状況を確認できます
        """
        AttendanceManager().confirm(sys._getframe().f_code.co_name)

    def open(self):
        """
        dakoker open:           MFクラウド勤怠ページを開きます
        """
        AttendanceManager(headless=False).open()
        # ブラウザを永続的に表示させるため、終了コマンドが実行されるまで待機
        input()

    def clear(self):
        """
        dakoker clear:          ユーザーログイン情報のローカルキャッシュをクリアします
        """
        UserInfoManager.remove_with_message()


def main():
    fire.Fire(Dakoker)



# coding:utf-8
from pick import pick
from halo import Halo
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import TimeoutException

from src.utils.color import Color
from src.user_info_manager import UserInfoManager


class Browser(object):
    TIMEOUT = 3
    ROOT_URL = "https://attendance.moneyforward.com"
    LOGIN_URL = ROOT_URL + "/employee_session/new"
    MYPAGE_URL = ROOT_URL + "/my_page"
    ATTENDANCE_URL = MYPAGE_URL + "/attendances"
    LOGIN_SUCCEED = "Login Success."
    LOGIN_FAILED = "Login Failed."

    DRIVER = 'DRIVER'
    SAFARI = 'Safari Drievr'
    CHROME = 'Chrome Drievr'

    def __init__(self, headless=True):
        self.headless = headless
        self.info_manager = UserInfoManager()
        self.setup_userinfo()

    def setup_userinfo(self):
        self.userinfo = self.info_manager.get()
        self.set_driver_to_userinfo()
        self.set_driver()

    def set_driver(self):
        if self.userinfo[self.DRIVER] == self.SAFARI:
            self.driver = webdriver.Safari()
        else:
            options = webdriver.ChromeOptions()
            options.headless = self.headless
            self.driver = webdriver.Chrome(chrome_options=options)

    def set_driver_to_userinfo(self):
        if self.DRIVER not in self.userinfo.keys():
            message = 'Please select your browser driver:'
            options = [self.SAFARI, self.CHROME]
            option, _ = pick(options, message)
            self.userinfo[self.DRIVER] = option

    def login(self):
        spinner = Halo(text='Loading login page...', spinner='dots')
        spinner.start()
        self.driver.get(self.LOGIN_URL)
        spinner.succeed("Login page loaded.")

        spinner = Halo(text='Login...', spinner='dots')
        spinner.start()
        return self.login_with_userinfo(spinner)

    def login_with_userinfo(self, spinner):
        self.driver.find_element_by_id(
            "employee_session_form_office_account_name"
        ).send_keys(self.userinfo[self.info_manager.CORP_ID])
        self.driver.find_element_by_id(
            "employee_session_form_account_name_or_email"
        ).send_keys(self.userinfo[self.info_manager.USER_ID])
        self.driver.find_element_by_id(
            "employee_session_form_password"
        ).send_keys(self.userinfo[self.info_manager.USER_PASS])

        self.driver.find_element_by_class_name(
            "attendance-before-login-card-button"
        ).click()

        return self.check_login(spinner)

    def check_login(self, spinner):
        try:
            WebDriverWait(self.driver, self.TIMEOUT).until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "attendance-card-title")
                )
            )
            self.info_manager.save(self.userinfo)
            spinner.succeed(self.LOGIN_SUCCEED)
            return True

        except TimeoutException:
            if self.driver.find_elements(By.CLASS_NAME, "is-error") != 0:
                Color.print(
                    Color.RED,
                    "\nCompany ID, User ID or Password is wrong."
                )
                spinner.fail(self.LOGIN_FAILED)
                UserInfoManager.remove()
                return False
            else:
                Color.print(Color.RED, "\nLogin Timeout.")
                spinner.fail(self.LOGIN_FAILED)
                return False

    def open_attendance(self):
        if self.login():
            spinner = Halo(text='Loading attendance page...', spinner='dots')
            self.driver.get(self.ATTENDANCE_URL)

        try:
            WebDriverWait(self.driver, self.TIMEOUT).until(
                EC.presence_of_element_located(
                    (By.CLASS_NAME, "modal-controller-my-page-attendances")
                )
            )
            spinner.succeed('Attendance page loaded.')
            return True

        except TimeoutException:
            return False



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['src', 'src.utils']

package_data = \
{'': ['*']}

install_requires = \
['better_exceptions>=0.2.2,<0.3.0',
 'bs4>=0.0.1,<0.0.2',
 'fire>=0.3.1,<0.4.0',
 'halo>=0.0.29,<0.0.30',
 'ipython>=7.15.0,<8.0.0',
 'keyring>=21.2.1,<22.0.0',
 'pick>=0.6.7,<0.7.0',
 'requests>=2.24.0,<3.0.0',
 'selenium>=3.141.0,<4.0.0']

entry_points = \
{'console_scripts': ['dakoker = src.dakoker:main']}

setup_kwargs = {
    'name': 'dakoker',
    'version': '0.1.25',
    'description': '',
    'long_description': '[![PyPI](https://img.shields.io/pypi/v/dakoker.svg)](https://pypi.python.org/pypi/dakoker)\n\nMF-Dakoker\n=======\n\n[MFクラウド勤怠](https://biz.moneyforward.com/attendance)利用者向けに作った打刻・勤怠状況確認ツールです。\n\n主な機能\n- MFクラウド勤怠へのログイン\n- 出勤・退勤の打刻\n- 休憩開始・終了の打刻\n- 当日の勤怠状況の確認(打刻日時)\n- ブラウザでMFクラウド勤怠ページを直接開く\n- ログイン情報キャシュ・キャッシュクリア\n\n実装したい機能\n- 二重打刻の防止機能\n- 指定日の勤怠状況の確認(打刻日時)\n\n動作環境\n- Python 3.8\n- poetry 1.0.9\n\n## インストール方法\n`pip3 install dakoker`\n\nまた、現在Chrome Driver・Safari Driverに対応しており、以下の設定が必要です。\n\n- Chrome Driverの場合\n  - `brew install chromedriver` (mac OS Xの場合) でchromedriverをインストール\n- Safari Driverの場合\n  - Safari 10.0以上を使っている場合使用可能です\n\n### 初回利用時\nログインのため、以下の情報を入力します。\n\n2回目以降は使用OSのパスワード保存領域(e.g. mac OS Xであればキーチェーン上)・その他ローカル領域にキャッシュされたログイン情報を読み込み、自動ログインします。\n\n- 企業ID\n- ユーザーID もしくは登録メールアドレス\n- パスワード\n\n![初回ログイン時](https://gyazo.com/e0657a3eecfc6a486a469a0cebd98db1.png)\n\n## 使い方\n\n- 出勤\n  - `dakoker start`\n- 退勤\n  - `dakoker end`\n- 休憩開始\n  - `dakoker start_break`\n- 休憩終了\n  - `dakoker end_break`\n- 当日の勤怠状況の確認\n  - `dakoker history`\n- ブラウザでMFクラウド勤怠ページを直接開く\n  - `dakoker open`\n- ログイン情報キャッシュのクリア\n  - `dakoker clear`\n',
    'author': 'nixiesquid',
    'author_email': 'audu817@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
